import { useState } from '../../../hooks/common/useState';
import { UserData } from 'src/types/auth';
import { onBeforeMount } from 'vue';
import Taro from '@tarojs/taro'

export function useAuthStore() {
  const [token, setToken] = useState('');
  const [user, setUser]   = useState<UserData>(null);

  onBeforeMount( () => {
    const token = Taro.getStorageSync<string>('token');
    const user = Taro.getStorageSync<UserData>('user');
    setToken(token);
    setUser(user);
  })

  const saveToken = (token: string) => {
    setToken(token);
    Taro.setStorageSync('token', token);
  }

  const saveUserInfo = (user: UserData) => {
    setUser(user);
    Taro.setStorageSync('user', user)
  }

  return {
    token,
    user,
    saveToken,
    saveUserInfo
  }
}
