import { BehaviorSubject, Observable, Subject } from 'rxjs';
import { filter, tap } from 'rxjs/operators';
import { useState } from '../../../hooks/common/useState';
import { onMounted } from 'vue';
import * as AgoraMiniappSDK from '../../../libs/Agora_SDK_for_WeChat';

interface VideoItemData {
  url: string;
  uid?: string;
  rotation?: string;
}

AgoraMiniappSDK.LOG.onlog = (text) => {
  console.log(text);
  };

const AGORA_APPID = '0a67966ff4ae4eb3b32446df0151e16a';
const streamLog$  = new Subject<string>();
const client      = new AgoraMiniappSDK.Client();
const agoraInit$  = 
  new Observable<boolean>( observer => {
    client.init(AGORA_APPID, () => {
      streamLog$.next('初始化成功')
      observer.next(true); 
      observer.complete();
    }, (e: any) => observer.error(e));
  })

export function useAgoraClient() {
  
  const channelState$ = new BehaviorSubject<string>('leave');
  
  const [streamList, setStreamList] = useState<VideoItemData[]>([])
  
  onMounted( () => {
    streamLog$.subscribe(console.log);
    subscribeRemoteStream(); 
  })

  const joinChannel = (ctoken: string, cid: string, uid: string) => {    
    agoraInit$    
      .pipe(tap(_=>console.log('准备加入频道')))
      .subscribe(
        _ => client.join(ctoken, cid, uid, () => {
          channelState$.next('joined');
          streamLog$.next('频道加入成功'+ctoken);          
          pushLocalStream();                    
        }, console.log),
        console.log
      )    
  }  

  const leaveChannel = () => {
    client.leave( () => channelState$.next('leave'))
  }

  const subscribeRemoteStream = () => {
    
    client.on("update-url", e => {
      console.log('genx', e)
    })

    client.on("stream-added", (e: any) => {
      console.log('新人加入', e)
      const uid = e.uid
      client.subscribe(
        uid,
        (url: string) => {
          const newStream = {url, uid}
          setStreamList(streamList.value.concat([newStream]))
        }
      );
    });

    client.on("stream-removed", e => {
      console.log('有人退出', e)
      const uid = e.uid;
      const index = streamList.value.findIndex( item => item.uid === uid );
      const list = streamList.value.splice(index, 1);
      setStreamList(list);
    })
  }

  const pushLocalStream = () => {
    channelState$
      .pipe(
        filter( state => state === 'joined' )
      )
      .subscribe( _ => {
        console.log('推送本地流')
        client.publish( (url: string) => {
          console.log('推送成功', url)
          const list = streamList.value.concat([{ url }])
          setStreamList(list)
        } )
      })
  }



  return {
    streamList,
    joinChannel,
    leaveChannel,
    pushLocalStream
  }
}