import Taro from '@tarojs/taro';
import { UserData } from "src/types/auth";
import { LoginData } from "src/types/user";
import { watch } from "vue";
import { useFormModel } from "../common/useFormModel";
import { useInjector } from "../common/useInjector";
import { useAuthStore } from "../store/auth/useAuthStore";

const loginTips = {
  successTip: '登录成功',
  loadingTip: '登录中...'
}

export function useLoginForm() {
  const [forms, action, states, res ] = useFormModel<LoginData, UserData>('/loginIn', {}, loginTips);
  const authStore = useInjector(useAuthStore, 'root');

  watch(res, val => {
    authStore.saveToken(val.token);
    authStore.saveUserInfo(val);
  })

  watch(states.httpStatus, val => {
    if(val === 'success') {
      Taro.navigateTo({url: '/pages/meeting/index'})
    }
  })

  return {
    forms,
    action
  }
}